#!/usr/bin/perl

# mergeGDASfiles.pl
# Copyright(c) 2005 	Franz Rueschendorf
# Created 4-Jan-2005

use Cwd;
use strict;
use File::Copy;
use File::Basename;

###############################  V A R I A B L E S  ############################
my $work_dir=cwd();
my $datum = scalar localtime;
my %allsamples=();
my %allmarkers=();
my %allsampleIDs=();
my %gtypes=();
my $outfile="merged_genotypes.txt";

if ($#ARGV < 0) { print "\nusage: perl \[path\]\\mergeGDASfiles.pl <file1> <file2>\n";exit;}

print "\nMerge GDAS export files ...\n\n";

my $file1="$ARGV[0]";		# GDAS export file
my $file2="$ARGV[1]";		# Gdas export file

my $file=$file1;
&ReadInfile;
$file=$file2;
&ReadInfile;

&WriteFile;

###############################  Read Genotypes  ###############################
sub ReadInfile {

	my %col_sample=();
	my $row="";
	my $c1=0;	# n-samples
	my $c2=0;	# n-markers
	my $sample="";
	my %localsamples=();
	my $col_affyid="";

	if (! -f $file) {
		print "\nno GDAS genotype file $file\n";
		exit;
	}
	print "Read genotypes:   $file\n";

	open (IN, "$file") || print "could not open GDAS file $file :$!\n";
	while (<IN>){
		next if /^$/;
		next if /^MPAM/;
		next if /Mapping Analysis/;

		chomp();
		my @v =split('\t',$_);
		if ($_ =~ /SNP ID/){
			for (my $i=0;$i<=$#v;$i++){
				if ($v[$i] eq "SNP ID") {$col_affyid=$i;}
				elsif ($v[$i] =~ /_Call/){
					$col_sample{$v[$i]}=$i;	# spalte des sample
					$c1++;
					my @row =split('_',$v[$i]);
					if ($#row > 1) { $sample= $row[2];}
					else{ $sample= $row[0];}
					if ($allsampleIDs{$sample}) {print "$sample ist schon vorhanden\n";}
					$allsampleIDs{$sample}++;
					
					$allsamples{$v[$i]}++;
					$localsamples{$v[$i]}++;
				}
				else {;}
			}
		}
		else {
			foreach my $key (keys %localsamples){
				$gtypes{$key}{$v[$col_affyid]}=$v[$col_sample{$key}];
			}
			$allmarkers{$v[$col_affyid]}++;
			$c2++;
		}
	}
	close (IN);
	print "\t$c2 markers\t$c1 samples\n\n";
}

############################   W R I T E   F I L E    #########################
sub WriteFile {
	my $snp="";
	my $sample="";
	my $c1=0;
	my $c2=0;
	
	open (OUT, "> $outfile") || print "could not open outfile $outfile :$!\n";
	print "Write outfile $outfile\n";
	print OUT "Affy SNP ID";
	foreach $sample (keys %allsamples){
		print OUT "\t$sample";
		$c1++;
	}
	print OUT "\n";

	foreach $snp (keys  %allmarkers) {
		print OUT "$snp";	
		foreach $sample (keys %allsamples){
			if ($gtypes{$sample}{$snp}){
				print OUT "\t$gtypes{$sample}{$snp}";
			}
			else {print OUT "\tNoCall";
			}
		}
		print OUT "\n";
		$c2++;	
	}	
	print "$c2 markers from $c1 samples are merged\n";
}

####################################  E N D  ###################################