#!/usr/bin/perl

#
# merlin_npl.pl :  Prepares Merlin output for displaying with gnuplot
# Syntax    :  merlin_npl.pl [-h] [-clean]
# Infiles   :  merlin.out (default)
# Outfiles  :  mynpl.gp, chr_line.plt, npl.plt and merlin_npl.ps
# 
# Created 15-Jan-2003 FR 
# Modified 15-Sept-2004 FR

use Cwd;

if ($ARGV[0] =~ "-h" ) { &Help;}
if ($ARGV[0] =~ "-v" ) { &Help;}
if ($ARGV[0] =~ "-clean" ) { &clean;}

print "\n\n";
print " * * * * * * * * * * * *  MERLIN_NPL.PL v1.1 * * * * * * * * * * * * *\n";
print " * * * * * * * * * View Merlin results with GNUPLOT  * * * * * * * * *\n\n";
print "Please wait ...\n\n";

my $maxlines = 999999; 	# Maximum number of lines (input-file)

my $xsum = 0.0;
my $xmin = 0;

my $ymax = 2;
my $ymin = -1;
my $lastxsum=0.;
my @xsums = ();
my %chrlabel=();
my $pfad = cwd();
my $datum = scalar localtime;

open (OUT1,"> npl.plt") || die "could not open npl.plt :$!\n";
open (OUT4,"> lod.plt") || die "could not open lod.plt :$!\n";

foreach $dir (c01,c02,c03,c04,c05,c06,c07,c08,c09,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22,c23,c24) {
	if (-f "$dir/merlin.out") {
		$in_file = "$dir/merlin.out";
		print "File: $in_file "; 
		&readnpl;

		my $xpos=(($lastxsum + $xsum)/2);
		my $var=$dir; $var =~ s/c0//;$var =~ s/c//;
		if ($var eq "23"){$var = "X";}
		if ($var eq "24"){$var = "XY";}
		if ($var > 9){ $xpos -=30;}
		else { $xpos -=5;}
		$chrlabel{$var}=$xpos;	  
		$lastxsum=$xsum;

		push (@xsums,$xsum);
	}
	else { print "$dir/merlin.out not existing \n"; }
} # end foreach
close (OUT1);
close (OUT4);

&print_chr_line;
&pr_batch;

# print "\nYou can now view results by \"gnuplot mynpl.gp\" \n\n";
system "gnuplot  mynpl.gp";

&clean;

###-----------------------  P R I N T  G N U P L O T  B A T C H   -----------------------###
sub pr_batch {
	
	my $titel = "$pfad - Merlin non-parametric analysis - $datum";
	open (OUT2,"> mynpl.gp") || die "could not open mynpl.gp :$!\n";

	print OUT2 "set autoscale\n";
	print OUT2 "set title \"$titel\\n\"\n" ;
	print OUT2 "set xlabel \"cM\"\n";
	print OUT2 "set ylabel \"Zmean/LOD*\"\n";
	my $ypos = $ymax + (($ymax-$ymin)/14);
	foreach (keys %chrlabel) {print OUT2 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	print OUT2 "set size 1,0.6\n";
	print OUT2 "set terminal postscript color 11\n";
	print OUT2 "set output \"merlin_npl.ps\"\n";
	print OUT2 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'npl.plt\' with lines,\'lod.plt\' with lines, \'chr_line.plt\' with lines\n";
	print OUT2 "set terminal x11\n";
	print OUT2 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]  \'npl.plt\' with lines, \'lod.plt\' with lines, \'chr_line.plt\' with lines\n";
	print OUT2 "\npause -1 \"Hit return to continue\"\n\n";

	close (OUT2);
}

######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {
	$ymax += 1;			# y-achse auf ganze NPL-zahl bringen
	my @buff = split('\.',$ymax);
	$ymax=$buff[0];

	open (OUT3,"> chr_line.plt") || die "could not open chr_line.plt :$!\n";

	for (my $i=1; $i<$ymax;$i++) {	# Linien bei LODs = 1,2,3,4,..
		print OUT3 "0 $i\n$xsum $i\n\n";
	}
	foreach (@xsums){	# Linien Chromosomenende 1,2,3,4,...22,x,xy
		print OUT3 "$_ $ymin \n$_ $ymax \n\n";         
	}
	close (OUT3);
}

#######################  R E A D   N P L - S C O R E S  #######################

sub readnpl {
	open (infile, "$in_file") || die "could not open $in_file :$!\n";

	$jetzt = $maxlines;
	$xmax = 0.0;
	my $hit=0;

	while (<infile>){
	@feld =	split(' ',$_);
	$count++;
		if (($feld[0] =~ /Pheno/) and ($feld[1] =~ /TRAIT/))   { 
			$score=$feld[2];
			print "  score: $score\n";
			$hit=1;
		}
		if ($feld[0] =~ /Skipping/) {print "\t$_\n";}

		if ($feld[0] =~ /max/)   { $jetzt = $count;}

		if ($#feld < 1) { $jetzt = $maxlines;}

		if ($count > ($jetzt)) {
		printf(OUT1 "%f %s\n", ($feld[0] + $xsum), $feld[1]);
		printf(OUT4 "%f %s\n", ($feld[0] + $xsum), $feld[4]);
		if ($feld[0] > $xmax) { $xmax = $feld[0];}  
		if ($feld[1] > $ymax) { $ymax = $feld[1];}
		}
	} # end while
	if ($hit == 0) {print "  no analysis\n";}
	$xsum = $xsum + $xmax;
	printf OUT1 "\n";
	printf OUT4 "\n";

  close (infile);

} ## end readnpl


##################################  H E L P  ############################
sub Help {
print "\n\n";
print " * * * * * * * * * * * * * MERLIN_NPL.PL v1.1  * * * * * * * * * * * *\n";
print " * * * * * * * * * View Merlin results with GNUPLOT  * * * * * * * * *\n";
print " * * * * * * * *  Created by Franz Rueschendorf  2003  * * * * * * * *\n";
print " * * * * * * * * * *  Max-Delbrueck-Center, Berlin * * * * * * * * * *\n";
print " * * * * * * * * * *  email: fruesch\@mdc-berlin.de * * * * * * * * * *\n\n";
print "Usage        : merlin_npl.pl [-h] [-clean]\n";
print "Infiles      : merlin.out (default)\n";
print "Outfiles     : mynpl.gp, npl.plt, lod.plt, chr_line.plt, merlin_npl.ps\n" ;
print "View results : gnuplot mynpl.gp\n\n";
exit 0;
}

##################################  C L E A N  #################################
sub clean { 
unlink glob("*.plt");
unlink glob("*.gp");
exit 0;
}

### end merlin_npl.pl
